#!/usr/bin/perl

##############################################################################
## setid - part of setiherder (http://www.sackheads.org/setiherder/         ##
## (C) Brian L. Naylor, 1999                                                ##
##                                                                          ##
## This script runs in the background, listens for setiherder on a          ##
## configurable port, execs setiathome, and delivers the output back to     ##
## the setiherder gui.  See docs/README.setid in the setiherder             ##
## distribution for more information.                                       ##
##                                                                          ##
## setid is licensed under the GNU GPL.                                     ##
##############################################################################

##############################################################################
# The simple communication protocol used here is documented in src/shnet.h   #
#                                                                            #
# PERL ENTHUSIASTS BEWARE:                                                   #
# I am a C coder.  I write C code.  Even when I write perl code, I tend to   #
# write C code.  I don't wanna hear about how ugly my perl is, I like it     #
# that way.  All you freaky perl trolls go bother someone else! :)           #
##############################################################################

require "Comm.pl" unless defined &Comm'init;
&Comm'init;

# versions
$sh_version     = "0.3";
$sh_net_version = "1.1";

#
# defaults
#
$port        = 7501;                 # tcp port to listen on
$setidir     = '/usr/local/seti/';   # where does seti@home live?
$seticlient  = 'setiathome';         # name of seti@home binary
$setiparam   = '-verbose';           # needed for seti@home v3 
$config      = 'user_info.txt';      # name of client config file
@allowhost   = ();                   # default to any
$passwd      = 'uSh0u|dcH4ng3Th1s';  # required if -w given - *please* do
                                     # not use a 'real' password for this,
                                     # as it is transmitted in the clear and
                                     # is easily sniffed.
$req_passwd  = 0;                    # do we require a password?

#
# important variables
#
@keeplines   = ();                   # client output to keep on detach
@lastlines   = ();                   # last $lastnum lines of client output
$lastnum     = 5;                    # *shrug*
$idledone    = 0;                    # how many units done while gui absent
$running     = 0;                    # is a client running? 
$detach      = 0;                    # should we detach if the gui goes away?
$connected   = 0;                    # currently have gui connected?
$len         = 255;                  # max buffer size

$sh_client   = 0;
$sh_pid      = -69;
$authed      = 0;
$seq         = -1;
$lastseq     = 0;

#
# "#defines"
#
$net_prefix       = 'SD:';

$cmd_start        = 'start';        # start seti@home client
$cmd_kill         = 'kill';         # kill seti@home client
$cmd_detach       = 'detach';       # please detach when gui exits
$cmd_ping         = 'ping';         # ping?
$cmd_idle         = 'idle';         # work done in gui absence
$cmd_auth         = 'auth';         # passwd issuance
$cmd_version      = 'version';      # version request

$err_ok           = 'success';      # success
$err_pong         = 'pong';         # pong
$err_active       = 'active';       # seti@home client is already running
$err_nactive      = 'nactive';      # seti@home client is not running
$err_connect      = 'connect';      # a setiherder is already connected
$err_client_start = 'client_start'; # couldn't start seti@home client
$err_client_comm  = 'client_comm';  # client is not responding
$err_host         = 'host';         # you are not allowed to connect
$err_died         = 'died';         # the seti client died
$err_shutdown     = 'shutdown';     # setid is being killed
$err_authreq      = 'authreq';      # what's the password?
$err_version      = 'version';      # this is my version
$err_confused     = 'confused';     # syntax error
$err_generic      = 'generic';      # generic error

require 5.002;
use Socket;

sub init_sock {
  socket(SH_LISTEN, PF_INET, SOCK_STREAM, 0)                    || die "socket: $!";
  setsockopt(SH_LISTEN, SOL_SOCKET, SO_REUSEADDR, pack('l', 1)) || die "setsockopt: $!";
  bind(SH_LISTEN, sockaddr_in($port, INADDR_ANY))               || die "bind: $!";
  listen(SH_LISTEN, SOMAXCONN)                                  || die "listen: $!";
}

sub debug($$) {
  my $severity = shift;
  my $msg = "@_";

  print "$msg\n";
  die "Fatal error.\n" if($severity == 0);
  return undef;
}

sub mychomp($) {
  chop($_[0]) if($_[0] =~ /\n$/);
  chop($_[0]) if($_[0] =~ /\r$/);
}

sub start_client {
  my $junk;

  if($running) {
    &send_client_text(1, "$err_active\n");
    return 1;
  }
  else {
    chop($setidir) if($setidir =~ /.+\/$/); # remove trailing /
    if(!chdir($setidir))
    {
      &send_client_text(1, "$err_generic Couldn't change to $setidir\n");
      return 1;
    }

    ($sh_client, $junk, $sh_pid) = &Comm'open_proc("./$seticlient $setiparam");
    $running = 1;
  }
  &send_client_text(1, "$err_ok\n");
  return undef; 
}

sub kill_client {
  if($running) {
    close($sh_client); 
    if( ! kill('TERM', $sh_pid)) {
      &send_client_text(0, "$err_generic Couldn't send TERM to child\n");
      if(kill("KILL", $sh_pid)) {
        &send_client_text(0, "$err_client_comm\n");
        return 1;
      }
    }
    $running = 0;
    &send_client_text(1, "$err_ok\n");
  }
  else {
    &send_client_text(1, "$err_nactive\n");
  }
}

sub parse_host_cmd {
  my $line = "@_";
  mychomp($line);

  $line =~ s/^SD://;
  $lastseq =  $line;
  $lastseq =~ s/^(\d+):.*$/$1/;
  $line =~ s/^\d+://;

  # insist upon authentication if necessary
  if($req_passwd && !$authed && ($line !~ /${cmd_auth}/))
  {
    &send_client_text(1, "$err_authreq\n");
    return undef;
  }
  
  if($line =~ /${cmd_start}/) {
    &start_client;
  }
  elsif($line =~ /${cmd_kill}/) {
    &kill_client;
  }
  elsif($line =~ /${cmd_detach} 1/) {
    $detach = 1;
    &send_client_text(1, "$err_ok\n");
  }
  elsif($line =~ /${cmd_detach} 0/) {
    $detach = 0;
    &send_client_text(1, "$err_ok\n");
  }
  elsif($line =~ /${cmd_ping}/)
  {
    &send_client_text(1, "$err_pong\n");
  }
  elsif($line =~ /${cmd_auth}/)
  {
    my $inpass = $line;
    $inpass =~ s/${cmd_auth} //;
    if($inpass eq $passwd)
    {
      $authed = 1;
      &send_client_text(1, "$err_ok\n");
    }
    else
    {
      &debug(1, "Invalid password.\n");
      sleep 5;
      &send_client_text(1, "$err_authreq\n");
    }
  }
  elsif($line =~ /${cmd_version}/)
  {
    &send_client_text(1, "$err_version $sh_version $sh_net_version\n");
  }  
  else 
  {
    &send_client_text(1, "$err_confused\n");
  }

  return undef;
}

sub send_client_text {
  my $isResponse = shift(@_);
  my $line = "@_";

  if($connected) {

    my($pref) = $net_prefix;
    if($isResponse == 1)
    {
      $pref .= "$lastseq:";
    }
    elsif($isResponse == 0)
    {
      $pref .= "$seq:";
      $seq--;
      $seq = -1 if($seq == -255);
    }
    elsif($isResponse == -1)
    {
      $pref = '';
    }
    else
    {
      print("send_client_text() got bogus first arg: $isResponse\n");
    }
  
    $line = "$pref$line";
    syswrite(SH_HOST, $line, $len);
  }
  return undef;
}

sub accept_conn {
  my $action = shift;
  local(*PEER) = @_;

  my $sock = getpeername(PEER);
  my $ip   = inet_ntoa((sockaddr_in($sock))[1]);
  my $host = gethostbyaddr(inet_aton($ip), AF_INET);

  if($? != 0)
  {
    &debug(1, "Error $? looking up hostname for $ip");
    $host = '[NULL]';
  }
  &debug(1, "Got connection from $host [$ip]");

  if($action == 0)  # reject
  { 
    print PEER "$err_connect\n";
    close(PEER);
  }
  else
  {
    if(@allowhost > 0)
    {
      my $hostmatch = 0;
      for(@allowhost)
      {
        $hostmatch =1 if(($ip eq $_) || ($host eq $_));
      }
      if(!$hostmatch)
      {
        &debug(1, "Host not in access list.  Denied.");
        sleep(5);
        &send_client_text(0, "$err_host\n");
        close(PEER);
        return undef;
      }
    }
    &summarize;
  }

  return undef;
}

sub summarize {
  &send_client_text(0, "$cmd_detach $detach\n");

  if($running) 
  {
    &send_client_text(0, "$cmd_idle $idledone\n");
    $idledone = 0;

    for(@keeplines) {
      &send_client_text(-1, $_);
    }

    for(@lastlines) {
      &send_client_text(-1, $_);
    }
  }
}

sub textgrok {
  my $in = "@_";

  # special case for when gui isn't connected
  if(!$connected && ($in =~ /Sending result/))
  {
    $idledone++;
    print "(DBG) Yerk!  Client finished a block while holmes is gone!\n";
  }

  if(scalar(@keeplines) < 25)
  {
    push(@keeplines, $in);
  }
  
  push(@lastlines, $in);
  shift(@lastlines) if scalar(@lastlines) > $lastnum;
}

sub signal_handler {
  my($signal) = @_;

  if( ($signal =~ /INT/) || ($signal =~ /TERM/) )
  {
    &send_client_text(0, "$err_shutdown\n");
    close(SH_HOST);
    &kill_client;
    exit(0);
  }  
  if($signal =~ /CHLD/)
  {
    my($wpid) = wait;
    if($wpid == $sh_pid) {
      $running = 0;
      close($sh_client);
      &send_client_text(0, "$err_died $?\n");
    }
    # /* else { ignore..  some random helper process.. } */
    # gotta reset this handler..  :-/
    $SIG{CHLD} = \&signal_handler;
  }

  return undef;
}


#
# let us begin (aka main())
#

# warn if root
if(!$< || !$>) {
  print STDERR 
  "\nPLEASE don't run this script as root.  There's no reason to do so, and\n" .
  "there are plenty of reasons *not* to.\n\nThis has been a public service " .
  "announcement.\n\n";
}

# parse args
while(@ARGV) 
{
  my $next = shift;
  if($next =~ /-p/) 
  {
    $port = int(shift);
    if($port == 0)
    {
      &debug(0, "-p requires an additional argument");
    }
  }
  elsif($next =~ /-d/)
  {
    $setidir = shift;
    if($setidir eq '')
    {
      &debug(0, "-d requires an additional argument");
    }
  }
  elsif($next =~ /-c/)
  {
    $seticlient = shift;
    if($seticlient eq '')
    {
      &debug(0, "-c requires an additional argument");
    }
  } 
  elsif($next =~ /-h/)
  {
    $next = shift;
    if($next eq '')
    {
      &debug(0, "-h requires an additional argument");
    }
    else
    {
      @allowhost = split(/,/, $next);
    }
  }
  elsif($next =~ /-w/)
  {
    $req_passwd = 1;
  }
  else 
  {
    print
      'Usage: setid [-p port][-d dir][-c client][-w][-h host1[,host2]]', "\n",
      "\t-p port  : override the default port (7501)\n",
      "\t-d dir   : override the default seti\@home directory (/usr/local/seti)",
      "\n",
      "\t-c client: override the default seti\@home binary name (setiathome)\n",
      "\t-h host..: list hosts allowed to connect\n",
      "\t-w       : require password\n",
      "\n";
    exit(1);
  }
}

#
# verify that what we were told is true
#

# directory exists
if ( ! -d "$setidir" )
{
  &debug(0, "$setidir is not a valid directory.\n");
}

# binary exists and is executable
if ( ! -x "$setidir/$seticlient" )
{
  &debug(0, "Cannot execute $setidir/$seticlient.\n");
}
if ( ! -f "$setidir/$seticlient"  || -l "$setidir/$seticlient" )
{
  &debug(1, "WARNING: $setidir/$seticlient is not a plain file!\n");
}

#
# install signal handlers
#
$SIG{CHLD} = \&signal_handler;
$SIG{INT}  = \&signal_handler;
$SIG{TERM} = \&signal_handler;


#
# initialize sockets
#
&init_sock;

#
# wait for connection
#
while(1)
{
  undef $select_bits;
  vec($select_bits, fileno(SH_LISTEN),  1) = 1;
  vec($select_bits, fileno(SH_HOST),    1) = 1; 
  vec($select_bits, fileno($sh_client), 1) = 1;
  if(select($rbits=$select_bits, undef, $ebits=$select_bits, undef))
  {
    # there's something to read, somewhere
    if(vec($rbits, fileno(SH_LISTEN), 1) &&     # master socket?
       ! vec($ebits, fileno(SH_LISTEN), 1))
    {
      if(!$connected) 
      {
        # no current connection
        accept(SH_HOST, SH_LISTEN);
        $connected = 1;
        &accept_conn(1, *SH_HOST);
      }
      else
      {
        # already connected
        accept(SH_REJECT, SH_LISTEN);
        &accept_conn(0, *SH_REJECT);
      }
    }
    elsif(vec($rbits, fileno(SH_HOST), 1) &&    # host socket?
          ! vec($ebits, fileno(SH_HOST), 1))
    {
      my($in) = '';
      $nread = sysread(SH_HOST, $in, $len);
      if($nread == 0 && eof(SH_HOST))
      { 
        print "Host closed connection.\n";
        $connected = 0;
        $seq = -1;
        $lastseq = 0;
        if(!$detach && $running)
        {
          &kill_client;
        }
        close(SH_HOST);
      }
      &parse_host_cmd($in);
    }
    elsif(vec($rbits, fileno($sh_client), 1) &&  # seti@home socket?
          ! vec($ebits, fileno($sh_client), 1))
    {
      my($in) = '';
      $nread = sysread($sh_client, $in, $len);
      if($nread > 0) {
        &send_client_text(-1, $in);
        &textgrok($in);
      }
      else {
        print "hmmm.\n";
        sleep 5;
      }
    }
  }
}


